{ Demo program for ChroMathsome - Linear Algebra.
  Below, you find example implementations of using functions available in
  ChroMathsome - Linear Algebra.

  In this demonstration program we will use matrices to store a table of values.
  We will try to compare the gain in speed between calculating values when we need them
  and looking up values in a table.
  As an example, we will calculate sin(x) + cos(x) for several values of x.
  Here's the background idea:
  - at program start, we create a table in memory (via a matrix) that holds the values
    for the sine and cosine of the values we need to calculate.
  - when we need a specific value for sin(x) or cos(x), we'll look it up in the matrix instead
    of calculating it.
  What's the deal you say?
  Well, reading values from a table (in memory) can speed up calculation-intensive procedures by more
  than 80% (yes, eighty)!
  Notice that for some functions the gain in speed is much greater (try ln(x) and log(x) -> 90%!, for example).
  Imagine the gain in speed when you need to calculate 1.000.000's of values!
  Note that you can create tables for all sorts of functions (log, ln, exp, sinh, ...) or
  even composite functions and why not ... just numbers you need often in your programs.
  Oh btw, the test machine was a P90. On faster machines, the speed gain may be less or even
  more impressive.

  Read the help file for information about the implemented functions
  and their limitations/possibilities!

  Author: John Merrit (john.merrit@net4all.be)
  Release date: January 30, 1998
  Reader may do with this code whatever he/she wants.

  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

  Brought to you by the letter "S" (for SSSSSSSpeed :-)).              }
unit speed;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, LinAlg,
  StdCtrls;

type
  TForm1 = class(TForm)
    Button1: TButton;
    Button2: TButton;
    Label1: TLabel;
    Label2: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
    { Private declarations }
  M: RealMatrix;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

{ Note: for this demo, we ignore the fact that the results (and angles) are in radians. }
procedure TForm1.FormCreate(Sender: TObject);
var i, j: integer;
begin
     InitRealMatrix(M, 2, 360);
     { Populate matrix M.
       Row 1 contains the sines and row2 the cosines of the values 1 to 360.
       It's a simple excercise to create a table containing all angles from 0 to 360 even
       with subdivisions (0.1, 0.01, ... degrees) -> do this yourself.                    }
     for i:= 1 to 2 do
     for j:= 1 to 360 do
     begin
     if i = 1 then M.Matrix^[i]^[j]:= sin(j)
     else M.Matrix^[i]^[j]:= cos(j);
     end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
     DestroyRealMatrix(M);
end;

{ Calculate the values when needed.
  This takes approximately 1 second.  }
procedure TForm1.Button1Click(Sender: TObject);
var i, j: integer;
    x: extended;
    t1: longint;
begin
     x:= 0;
     Form1.Caption:= 'Busy calculating ... please wait';
     t1:= GetTickCount; // Good old Windows function to measure time intervals.
     for j:= 1 to 1000 do
     begin
     for i:= 1 to 360 do
     x:= sin(i) + cos(i) + x;
     end;
     t1:= GetTickCount - t1;
     Label1.Caption:= 'Result: ' + FloatToStr(x) + ' Time used: ' + IntToStr(t1) + ' msec';
     Form1.Caption:= 'Done.';
end;

{ Now, we look up the values in the matrix instead of calculating them.
  This runs more than 80% faster than calculating the results!
  This code takes approximately 0.175 sec.                              }
procedure TForm1.Button2Click(Sender: TObject);
var i, j: integer;
    x: extended;
    t1: longint;
begin
     x:= 0;
     Form1.Caption:= 'Busy calculating ... please wait';
     t1:= GetTickCount;
     for j:= 1 to 1000 do
     begin
     for i:= 1 to 360 do
     x:= M.Matrix^[1]^[i] + M.Matrix^[2]^[i] + x;
     end;
     t1:= GetTickCount - t1;
     Label2.Caption:= 'Result: ' + FloatToStr(x) + ' Time used: ' + IntToStr(t1) + ' msec';
     Form1.Caption:= 'Done.';
end;

end.
